==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/re/re-tre.c <==
/*
   Copyright 2004 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#if WITH_TRE
static char const rcsid[] = 
"$Id: re-tre.c,v 1.2 2004/07/13 10:22:52 lwa Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <tre/regex.h>

#include "../ltdl.h"
#include "re.h"

typedef int (*regaexec_t) (const regex_t *preg, const char *string,
                           regamatch_t *match, regaparams_t params,
                           int eflags);
typedef int  (*regcomp_t) (regex_t *preg, const char *regex, int cflags);
typedef void (*regfree_t) (regex_t *preg);

typedef size_t (*regerror_t)(int errcode, const regex_t *preg, char *errbuf,
                             size_t errbuf_size);

typedef void (*regaparams_default_t)(regaparams_t *params);


regaexec_t tre_regaexec;
regcomp_t  tre_regcomp;
regfree_t  tre_regfree;
regerror_t tre_regerror;
regaparams_default_t tre_regaparams_default;

const char *re_tre_init(void) {
  lt_dlhandle dlh;
  if ( lt_dlinit() != 0 ) 
    return lt_dlerror();

  if ( (dlh = lt_dlopenext("libtre")) == NULL ) 
    return lt_dlerror();
    
  if ( (tre_regaexec = (regaexec_t)lt_dlsym(dlh, "regaexec")) == NULL )
    return lt_dlerror();

  if ( (tre_regcomp = (regcomp_t)lt_dlsym(dlh, "regcomp")) == NULL )
    return lt_dlerror();
    
  if ( (tre_regfree = (regfree_t)lt_dlsym(dlh, "regfree")) == NULL)
    return lt_dlerror();
    
  if ( (tre_regerror = (regerror_t)lt_dlsym(dlh, "regerror")) == NULL) 
    return lt_dlerror();

  if ( (tre_regaparams_default =
	(regaparams_default_t)lt_dlsym(dlh, "regaparams_default")) == NULL) 
    return lt_dlerror();

  return NULL;
}

static void re_close(RE *re) {
  if (re->code) {
    tre_regfree(re->code);
    free(re->code);
    re->code = NULL;
  }
  if (re->hint) {
    free(re->hint);
    re->hint = NULL;
  }
}

static int re_compile(RE *re, char *pattern, int icase) {
  int ret;
  re->code = malloc(sizeof (regex_t));
  if (re->code==NULL) {
    re->err = RE_ERR_MEMORY;
    re->errstr = "memory";
  } else {
    ret = tre_regcomp(re->code, pattern, re->options | (icase?REG_ICASE:0));
    if (ret != 0) {
      if (ret == REG_ESPACE) {
        re->err =  RE_ERR_MEMORY;
      } else {
        re->err =  RE_ERR_SYNTAX;
      }
      re->errstr = "some error";
    } else {
      re->errstr = NULL;
      re->err = 0;
    }
  }
  return re->err;
}


static int re_exec(RE *re, char *string) {
  regaparams_t regparams ;
  regamatch_t regamatch = {0,};
  int ret;

  memcpy(&regparams, re->hint, sizeof(regaparams_t));

#if 0
  regparams.cost_ins=1;
  regparams.cost_del=1;
  regparams.cost_subst=1;
  regparams.max_cost=1;
  regparams.max_ins=1;
  regparams.max_del=1;
  regparams.max_subst=1;
  regparams.max_err=1;
#endif

  ret = tre_regaexec(re->code, string, &regamatch, regparams, 0);
  return ret!=0;
}

char *re_tre_params(RE *re, char *s) {
  regaparams_t *params = malloc(sizeof(regaparams_t));
  int *counter;
  int end = 0;
  tre_regaparams_default(params);
  counter = &params->max_err;
  *counter = 1;
  printf("s=%s\n", s);
  while(*s && !end) {
    if (*s>='0' && *s<='9') {
      *counter = *s - '0';
    } else switch(*s) {
    case 'i': counter = &params->max_ins; break;
    case 'd': counter = &params->max_del; break;
    case 's': counter = &params->max_subst; break;
    default: end = 1; break;
    }
    s++;
  }
  re->hint = params;
  return s;
}

RE re_tre_extended = {
  re_compile,
  re_exec,
  re_close,
  REG_EXTENDED|REG_NOSUB,
  0,
};
#endif

==> lmtpd-0.10.4.tar.gz: lmtpd-0.10.4/re/re-tre.c <==
/*
   Copyright 2004 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

#if WITH_TRE

static char const rcsid[] UNUSED = 
"$Id: re-tre.c,v 1.4 2004/10/27 10:42:08 lwa Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <tre/regex.h>

#include "../ltdl.h"
#include "re.h"

typedef int (*regaexec_t) (const regex_t *preg, const char *string,
                           regamatch_t *match, regaparams_t params,
                           int eflags);
typedef int  (*regcomp_t) (regex_t *preg, const char *regex, int cflags);
typedef void (*regfree_t) (regex_t *preg);

typedef size_t (*regerror_t)(int errcode, const regex_t *preg, char *errbuf,
                             size_t errbuf_size);

typedef void (*regaparams_default_t)(regaparams_t *params);


regaexec_t tre_regaexec;
regcomp_t  tre_regcomp;
regfree_t  tre_regfree;
regerror_t tre_regerror;
regaparams_default_t tre_regaparams_default;

const char *re_tre_init(void) {
  lt_dlhandle dlh;
  if ( lt_dlinit() != 0 ) 
    return lt_dlerror();

  if ( (dlh = lt_dlopenext("libtre")) == NULL ) 
    return lt_dlerror();
    
  if ( (tre_regaexec = (regaexec_t)lt_dlsym(dlh, "regaexec")) == NULL )
    return lt_dlerror();

  if ( (tre_regcomp = (regcomp_t)lt_dlsym(dlh, "regcomp")) == NULL )
    return lt_dlerror();
    
  if ( (tre_regfree = (regfree_t)lt_dlsym(dlh, "regfree")) == NULL)
    return lt_dlerror();
    
  if ( (tre_regerror = (regerror_t)lt_dlsym(dlh, "regerror")) == NULL) 
    return lt_dlerror();

  if ( (tre_regaparams_default =
	(regaparams_default_t)lt_dlsym(dlh, "regaparams_default")) == NULL) 
    return lt_dlerror();

  return NULL;
}

static void re_close(RE *re) {
  if (re->code) {
    tre_regfree(re->code);
    free(re->code);
    re->code = NULL;
  }
  if (re->hint) {
    free(re->hint);
    re->hint = NULL;
  }
}

static int re_compile(RE *re, char *pattern, int icase) {
  int ret;
  re->code = malloc(sizeof (regex_t));
  if (re->code==NULL) {
    re->err = RE_ERR_MEMORY;
    re->errstr = "memory";
  } else {
    ret = tre_regcomp(re->code, pattern, re->options | (icase?REG_ICASE:0));
    if (ret != 0) {
      if (ret == REG_ESPACE) {
        re->err =  RE_ERR_MEMORY;
      } else {
        re->err =  RE_ERR_SYNTAX;
      }
      re->errstr = "some error";
    } else {
      re->errstr = NULL;
      re->err = 0;
    }
  }
  return re->err;
}


static int re_exec(RE *re, char *string) {
  regaparams_t regparams ;
  regamatch_t regamatch = {0,};
  int ret;

  memcpy(&regparams, re->hint, sizeof(regaparams_t));

#if 0
  regparams.cost_ins=1;
  regparams.cost_del=1;
  regparams.cost_subst=1;
  regparams.max_cost=1;
  regparams.max_ins=1;
  regparams.max_del=1;
  regparams.max_subst=1;
  regparams.max_err=1;
#endif

  ret = tre_regaexec(re->code, string, &regamatch, regparams, 0);
  return ret!=0;
}

char *re_tre_params(RE *re, char *s) {
  regaparams_t *params = malloc(sizeof(regaparams_t));
  int *counter;
  int end = 0;
  tre_regaparams_default(params);
  counter = &params->max_err;
  *counter = 1;
  printf("s=%s\n", s);
  while(*s && !end) {
    if (*s>='0' && *s<='9') {
      *counter = *s - '0';
    } else switch(*s) {
    case 'i': counter = &params->max_ins; break;
    case 'd': counter = &params->max_del; break;
    case 's': counter = &params->max_subst; break;
    default: end = 1; break;
    }
    s++;
  }
  re->hint = params;
  return s;
}

RE re_tre_extended = {
  re_compile,
  re_exec,
  re_close,
  REG_EXTENDED|REG_NOSUB,
  0,
};
#endif
